class BrokerEnvVars:
    BROKER_SERVER = "BROKER_SERVER"
    BROKER_PORT = "BROKER_PORT"
    BROKER_USERNAME = "BROKER_USERNAME"
    BROKER_PWD = "BROKER_PWD"
    BROKER_CLUSTER_SERVER = "BROKER_CLUSTER_SERVER"
